@@ -3,11 +3,11 @@
  *
  * See Copyright for the status of this software.
  *
- * $Id: entities.c,v 1.1 1998/08/13 03:39:04 veillard Exp $
+ * $Id: entities.c,v 1.10 1998/11/27 06:39:46 veillard Exp $
  */
 
 #include <stdio.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include "entities.h"
 
@@ -31,6 +31,9 @@ xmlEntitiesTablePtr xmlPredefinedEntities = NULL;
 
 /*
  * A buffer used for converting entities to their equivalent and back.
+ *
+ * TODO: remove this, this helps performances but forbid reentrancy in a 
+ *       stupid way.
  */
 static int buffer_size = 0;
 static CHAR *buffer = NULL;
@@ -47,7 +50,6 @@ void growBuffer(void) {
 /*
  * xmlFreeEntity : clean-up an entity record.
  */
-
 void xmlFreeEntity(xmlEntityPtr entity) {
     if (entity == NULL) return;
 
@@ -63,12 +65,13 @@ void xmlFreeEntity(xmlEntityPtr entity) {
 }
 
 /*
- * xmlAddDocEntity : register a new entity for an entities table.
+ * xmlAddEntity : register a new entity for an entities table.
  *
  * TODO !!! We should check here that the combination of type
  *          ExternalID and SystemID is valid.
  */
-static void xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
+static void
+xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
     int i;
     xmlEntityPtr cur;
@@ -116,8 +119,10 @@ static void xmlAddEntity(xmlEntitiesTablePtr table, const CHAR *name, int type,
     table->nb_entities++;
 }
 
-/*
- * Set up xmlPredefinedEntities from xmlPredefinedEntityValues.
+/**
+ * xmlInitializePredefinedEntities:
+ *
+ * Set up the predefined entities.
  */
 void xmlInitializePredefinedEntities(void) {
     int i;
@@ -138,16 +143,46 @@ void xmlInitializePredefinedEntities(void) {
 	out = &value[0];
 	for (;(*out++ = (CHAR) *in);)in++;
         xmlAddEntity(xmlPredefinedEntities, (const CHAR *) &name[0],
-	             XML_INTERNAL_GENERAL_ENTITY, NULL, NULL,
+	             XML_INTERNAL_PREDEFINED_ENTITY, NULL, NULL,
 		     &value[0]);
     }
 }
 
-
-/*
- * xmlAddDtdEntity : register a new entity for this DTD.
+/**
+ * xmlGetPredefinedEntity:
+ * @name:  the entity name
+ *
+ * Check whether this name is an predefined entity.
+ *
+ * return values: NULL if not, othervise the entity
  */
-void xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
+xmlEntityPtr
+xmlGetPredefinedEntity(const CHAR *name) {
+    int i;
+    xmlEntityPtr cur;
+
+    if (xmlPredefinedEntities == NULL)
+        xmlInitializePredefinedEntities();
+    for (i = 0;i < xmlPredefinedEntities->nb_entities;i++) {
+	cur = &xmlPredefinedEntities->table[i];
+	if (!xmlStrcmp(cur->name, name)) return(cur);
+    }
+    return(NULL);
+}
+
+/**
+ * xmlAddDtdEntity:
+ * @doc:  the document
+ * @name:  the entity name
+ * @type:  the entity type XML_xxx_yyy_ENTITY
+ * @ExternalID:  the entity external ID if available
+ * @SystemID:  the entity system ID if available
+ * @content:  the entity content
+ *
+ * Register a new entity for this document DTD.
+ */
+void
+xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
     xmlEntitiesTablePtr table;
 
@@ -163,10 +198,19 @@ void xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
     xmlAddEntity(table, name, type, ExternalID, SystemID, content);
 }
 
-/*
- * xmlAddDocEntity : register a new entity for this document.
+/**
+ * xmlAddDocEntity:
+ * @doc:  the document
+ * @name:  the entity name
+ * @type:  the entity type XML_xxx_yyy_ENTITY
+ * @ExternalID:  the entity external ID if available
+ * @SystemID:  the entity system ID if available
+ * @content:  the entity content
+ *
+ * Register a new entity for this document.
  */
-void xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
+void
+xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content) {
     xmlEntitiesTablePtr table;
 
@@ -178,11 +222,18 @@ void xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
     xmlAddEntity(doc->entities, name, type, ExternalID, SystemID, content);
 }
 
-/*
- * xmlGetDtdEntity : do an entity lookup in the Dtd entity hash table and
- *       returns the corrsponding entity, if found, NULL otherwise.
+/**
+ * xmlGetDtdEntity:
+ * @doc:  the document referencing the entity
+ * @name:  the entity name
+ *
+ * Do an entity lookup in the Dtd entity hash table and
+ * returns the corresponding entity, if found.
+ * 
+ * return values: A pointer to the entity structure or NULL if not found.
  */
-xmlEntityPtr xmlGetDtdEntity(xmlDocPtr doc, const CHAR *name) {
+xmlEntityPtr
+xmlGetDtdEntity(xmlDocPtr doc, const CHAR *name) {
     int i;
     xmlEntityPtr cur;
     xmlEntitiesTablePtr table;
@@ -197,12 +248,19 @@ xmlEntityPtr xmlGetDtdEntity(xmlDocPtr doc, const CHAR *name) {
     return(NULL);
 }
 
-/*
- * xmlGetDocEntity : do an entity lookup in the document entity hash table and
- *       returns the corrsponding entity, otherwise a lookup is done
- *       in the predefined entities too.
+/**
+ * xmlGetDocEntity:
+ * @doc:  the document referencing the entity
+ * @name:  the entity name
+ *
+ * Do an entity lookup in the document entity hash table and
+ * returns the corrsponding entity, otherwise a lookup is done
+ * in the predefined entities too.
+ * 
+ * return values: A pointer to the entity structure or NULL if not found.
  */
-xmlEntityPtr xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
+xmlEntityPtr
+xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
     int i;
     xmlEntityPtr cur;
     xmlEntitiesTablePtr table;
@@ -226,14 +284,36 @@ xmlEntityPtr xmlGetDocEntity(xmlDocPtr doc, const CHAR *name) {
 }
 
 /*
- * xmlEncodeEntities : do a global encoding of a string, replacing the
- *                     basic content with their entities form.
- * TODO !!!! rewite !!!
+ * [2] Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD]
+ *                  | [#x10000-#x10FFFF]
+ * any Unicode character, excluding the surrogate blocks, FFFE, and FFFF.
  */
-CHAR *xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
+#define IS_CHAR(c)							\
+    (((c) == 0x09) || ((c) == 0x0a) || ((c) == 0x0d) ||			\
+     (((c) >= 0x20) && ((c) != 0xFFFE) && ((c) != 0xFFFF)))
+
+/**
+ * xmlEncodeEntities:
+ * @doc:  the document containing the string
+ * @input:  A string to convert to XML.
+ *
+ * Do a global encoding of a string, replacing the predefined entities
+ * and non ASCII values with their entities and CharRef counterparts.
+ *
+ * TODO !!!! Once moved to UTF-8 internal encoding, the encoding of non-ascii
+ *           get erroneous.
+ *
+ * TODO This routine is not reentrant and this will be changed, the interface
+ *      should not be modified though.
+ * 
+ * return values: A newly allocated string with the substitution done.
+ */
+CHAR *
+xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
     const CHAR *cur = input;
     CHAR *out = buffer;
 
+    if (input == NULL) return(NULL);
     if (buffer == NULL) {
         buffer_size = 1000;
         buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
@@ -253,8 +333,6 @@ CHAR *xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
 
 	/*
 	 * By default one have to encode at least '<', '>', '"' and '&' !
-	 * One could try a better encoding using the entities defined and
-	 * used as a compression code !!!.
 	 */
 	if (*cur == '<') {
 	    *out++ = '&';
@@ -286,22 +364,58 @@ CHAR *xmlEncodeEntities(xmlDocPtr doc, const CHAR *input) {
 	    *out++ = 'o';
 	    *out++ = 's';
 	    *out++ = ';';
-	} else {
+	} else if (((*cur >= 0x20) && (*cur < 0x80)) ||
+	    (*cur == '\n') || (*cur == '\r') || (*cur == '\t')) {
 	    /*
 	     * default case, just copy !
 	     */
 	    *out++ = *cur;
+#ifndef USE_UTF_8
+	} else if ((sizeof(CHAR) == 1) && (*cur >= 0x80)) {
+	    char buf[10], *ptr;
+#ifdef HAVE_SNPRINTF
+	    snprintf(buf, 9, "&#%d;", *cur);
+#else
+	    sprintf(buf, "&#%d;", *cur);
+#endif
+            ptr = buf;
+	    while (*ptr != 0) *out++ = *ptr++;
+#endif
+	} else if (IS_CHAR(*cur)) {
+	    char buf[10], *ptr;
+
+#ifdef HAVE_SNPRINTF
+	    snprintf(buf, 9, "&#%d;", *cur);
+#else
+	    sprintf(buf, "&#%d;", *cur);
+#endif
+            ptr = buf;
+	    while (*ptr != 0) *out++ = *ptr++;
 	}
+#if 0
+	else {
+	    /*
+	     * default case, this is not a valid char !
+	     * Skip it...
+	     */
+	    fprintf(stderr, "xmlEncodeEntities: invalid char %d\n", (int) *cur);
+	}
+#endif
 	cur++;
     }
     *out++ = 0;
     return(buffer);
 }
 
-/*
- * xmlCreateEntitiesTable : create and initialize an enmpty hash table
+/**
+ * xmlCreateEntitiesTable:
+ *
+ * create and initialize an empty entities hash table.
+ *
+ * return values: the xmlEntitiesTablePtr just created or NULL in case of error.
  */
-xmlEntitiesTablePtr xmlCreateEntitiesTable(void) {
+xmlEntitiesTablePtr
+xmlCreateEntitiesTable(void) {
     xmlEntitiesTablePtr ret;
 
     ret = (xmlEntitiesTablePtr) 
@@ -324,10 +438,14 @@ xmlEntitiesTablePtr xmlCreateEntitiesTable(void) {
     return(ret);
 }
 
-/*
- * xmlFreeEntitiesTable : clean up and free an entities hash table.
+/**
+ * xmlFreeEntitiesTable:
+ * @table:  An entity table
+ *
+ * Deallocate the memory used by an entities hash table.
  */
-void xmlFreeEntitiesTable(xmlEntitiesTablePtr table) {
+void
+xmlFreeEntitiesTable(xmlEntitiesTablePtr table) {
     int i;
 
     if (table == NULL) return;
@@ -339,10 +457,70 @@ void xmlFreeEntitiesTable(xmlEntitiesTablePtr table) {
     free(table);
 }
 
-/*
- * Dump the content of an entity table to the document output.
+/**
+ * xmlCopyEntitiesTable:
+ * @table:  An entity table
+ *
+ * Build a copy of an entity table.
+ * 
+ * return values: the new xmlEntitiesTablePtr or NULL in case of error.
  */
-void xmlDumpEntitiesTable(xmlEntitiesTablePtr table) {
+xmlEntitiesTablePtr
+xmlCopyEntitiesTable(xmlEntitiesTablePtr table) {
+    xmlEntitiesTablePtr ret;
+    xmlEntityPtr cur, ent;
+    int i;
+
+    ret = (xmlEntitiesTablePtr) malloc(sizeof(xmlEntitiesTable));
+    if (ret == NULL) {
+        fprintf(stderr, "xmlCopyEntitiesTable: out of memory !\n");
+	return(NULL);
+    }
+    ret->table = (xmlEntityPtr) malloc(table->max_entities *
+                                         sizeof(xmlEntity));
+    if (ret->table == NULL) {
+        fprintf(stderr, "xmlCopyEntitiesTable: out of memory !\n");
+	free(ret);
+	return(NULL);
+    }
+    ret->max_entities = table->max_entities;
+    ret->nb_entities = table->nb_entities;
+    for (i = 0;i < ret->nb_entities;i++) {
+	cur = &ret->table[i];
+	ent = &table->table[i];
+	cur->len = ent->len;
+	cur->type = ent->type;
+	if (ent->name != NULL)
+	    cur->name = xmlStrdup(ent->name);
+	else
+	    cur->name = NULL;
+	if (ent->ExternalID != NULL)
+	    cur->ExternalID = xmlStrdup(ent->ExternalID);
+	else
+	    cur->ExternalID = NULL;
+	if (ent->SystemID != NULL)
+	    cur->SystemID = xmlStrdup(ent->SystemID);
+	else
+	    cur->SystemID = NULL;
+	if (ent->content != NULL)
+	    cur->content = xmlStrdup(ent->content);
+	else
+	    cur->content = NULL;
+    }
+    return(ret);
+}
+
+/**
+ * xmlDumpEntitiesTable:
+ * @table:  An entity table
+ *
+ * This will dump the content of the entity table as an XML DTD definition
+ *
+ * NOTE: TODO an extra parameter allowing a reentant implementation will
+ *       be added.
+ */
+void
+xmlDumpEntitiesTable(xmlEntitiesTablePtr table) {
     int i;
     xmlEntityPtr cur;
 
