@@ -3,13 +3,13 @@
  *
  * See Copyright for the status of this software.
  *
- * $Id: entities.h,v 1.1 1998/08/13 03:39:04 veillard Exp $
+ * $Id: entities.h,v 1.4 1998/11/27 06:39:47 veillard Exp $
  */
 
 #ifndef __XML_ENTITIES_H__
 #define __XML_ENTITIES_H__
-#include "parser.h"
 
+#include "parser.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -20,6 +20,7 @@ extern "C" {
 #define XML_EXTERNAL_GENERAL_UNPARSED_ENTITY	3
 #define XML_INTERNAL_PARAMETER_ENTITY		4
 #define XML_EXTERNAL_PARAMETER_ENTITY		5
+#define XML_INTERNAL_PREDEFINED_ENTITY		6
 
 /*
  * An unit of storage for an entity, contains the string, the value
@@ -45,9 +46,10 @@ typedef struct xmlEntity {
 typedef struct xmlEntitiesTable {
     int nb_entities;		/* number of elements stored */
     int max_entities;		/* maximum number of elements */
-    xmlEntityPtr table;		/* the table of entities */
+    xmlEntityPtr table;	        /* the table of entities */
 } xmlEntitiesTable, *xmlEntitiesTablePtr;
 
+
 /*
  * External functions :
  */
@@ -56,12 +58,17 @@ extern void xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content);
 extern void xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content);
+extern xmlEntityPtr xmlGetPredefinedEntity(const CHAR *name);
 extern xmlEntityPtr xmlGetDocEntity(xmlDocPtr doc, const CHAR *name);
 extern xmlEntityPtr xmlGetDtdEntity(xmlDocPtr doc, const CHAR *name);
 extern CHAR *xmlEncodeEntities(xmlDocPtr doc, const CHAR *input);
 extern xmlEntitiesTablePtr xmlCreateEntitiesTable(void);
+extern xmlEntitiesTablePtr xmlCopyEntitiesTable(xmlEntitiesTablePtr table);
 extern void xmlFreeEntitiesTable(xmlEntitiesTablePtr table);
 extern void xmlDumpEntitiesTable(xmlEntitiesTablePtr table);
+extern xmlParserInputPtr xmlNewEntityInputStream(xmlParserCtxtPtr ctxt,
+                                                 xmlEntityPtr entity);
+extern xmlEntitiesTablePtr xmlCopyEntitiesTable(xmlEntitiesTablePtr table);
 
 #ifdef __cplusplus
 }
